# coding=utf-8

def get_version():
    return '%s.%s' % (VERSION[0], VERSION[1])


VERSION = (0, 24,)

__version__ = get_version()
__author__ = 'Javier Cordero'
__email__ = 'jcorderomartinez@gmail.com'
__license__ = 'Apache 2.0'

from .smsfactory import SMSFactory

smsfactory = SMSFactory()

default_app_config = 'mailfactory_extras.apps.MailFactoryExtrasConfig'



# The default ``config.py``


def set_prefs(prefs):
    """This function is called before opening the project."""

    # Specify which files and folders to ignore in the project.
    # Changes to ignored resources are not added to the history and
    # VCSs.  Also they are not returned in `Project.get_files()`.
    # Note that ``?`` and ``*`` match all characters but slashes.
    # '*.pyc': matches 'test.pyc' and 'pkg/test.pyc'
    # 'mod*.pyc': matches 'test/mod1.pyc' but not 'mod/1.pyc'
    # '.svn': matches 'pkg/.svn' and all of its children
    # 'build/*.o': matches 'build/lib.o' but not 'build/sub/lib.o'
    # 'build//*.o': matches 'build/lib.o' and 'build/sub/lib.o'
    prefs['ignored_resources'] = ['*.pyc', '*~', '.ropeproject',
                                  '.hg', '.svn', '_svn', '.git']

    # Specifies which files should be considered python files.  It is
    # useful when you have scripts inside your project.  Only files
    # ending with ``.py`` are considered to be python files by
    # default.
    #prefs['python_files'] = ['*.py']

    # Custom source folders:  By default rope searches the project
    # for finding source folders (folders that should be searched
    # for finding modules).  You can add paths to that list.  Note
    # that rope guesses project source folders correctly most of the
    # time; use this if you have any problems.
    # The folders should be relative to project root and use '/' for
    # separating folders regardless of the platform rope is running on.
    # 'src/my_source_folder' for instance.
    #prefs.add('source_folders', 'src')

    # You can extend python path for looking up modules
    #prefs.add('python_path', '~/python/')

    # Should rope save object information or not.
    prefs['save_objectdb'] = True
    prefs['compress_objectdb'] = False

    # If `True`, rope analyzes each module when it is being saved.
    prefs['automatic_soa'] = True
    # The depth of calls to follow in static object analysis
    prefs['soa_followed_calls'] = 0

    # If `False` when running modules or unit tests "dynamic object
    # analysis" is turned off.  This makes them much faster.
    prefs['perform_doa'] = True

    # Rope can check the validity of its object DB when running.
    prefs['validate_objectdb'] = True

    # How many undos to hold?
    prefs['max_history_items'] = 32

    # Shows whether to save history across sessions.
    prefs['save_history'] = True
    prefs['compress_history'] = False

    # Set the number spaces used for indenting.  According to
    # :PEP:`8`, it is best to use 4 spaces.  Since most of rope's
    # unit-tests use 4 spaces it is more reliable, too.
    prefs['indent_size'] = 4

    # Builtin and c-extension modules that are allowed to be imported
    # and inspected by rope.
    prefs['extension_modules'] = []

    # Add all standard c-extensions to extension_modules list.
    prefs['import_dynload_stdmods'] = True

    # If `True` modules with syntax errors are considered to be empty.
    # The default value is `False`; When `False` syntax errors raise
    # `rope.base.exceptions.ModuleSyntaxError` exception.
    prefs['ignore_syntax_errors'] = False

    # If `True`, rope ignores unresolvable imports.  Otherwise, they
    # appear in the importing namespace.
    prefs['ignore_bad_imports'] = False


def project_opened(project):
    """This function is called after opening the project."""
    # Do whatever you like here!



# coding=utf-8

""" :synopsis: Utilities """


from mail_factory import factory
from django.utils import six

def registered_mails_names():
    if hasattr(factory, '_registry'):
        for k, v in six.iteritems(factory._registry):
            yield k, v.__name__
        return
    else:
        for k, v in six.iteritems(factory.mail_map):
            yield k, v.__name__
        return




# coding=utf-8

""":synopsis: Just a simple version of the original Factory object but using SMS

"""

from mail_factory import exceptions


class SMSFactory(object):
    def __init__(self, *args, **kwargs):
        self.sms_map = {}

    def register(self, sms_klass):
        if not hasattr(sms_klass, 'template_name'):
            raise exceptions.MailFactoryError(
                '{0} needs a template_name parameter to be registered'.format(
                    sms_klass.__name__))

        self.sms_map[sms_klass.template_name] = sms_klass

    def unregister(self, sms_klass):
        if not hasattr(sms_klass, 'template_name'):
            raise exceptions.MailFactoryError(
                '{0} needs a template_name parameter to be registered'.format(
                    sms_klass.__name__))

        del sms_klass[sms_klass.template_name]

    def send(self, template_name, to_phone, context):
        return self.sms_map[template_name](context=context).send(to_phone)



# coding=utf-8

import os

from abc import ABCMeta, abstractmethod

from django.utils import six
from django.template import TemplateDoesNotExist

from mail_factory.mails import BaseMail


class BaseSMS(BaseMail, six.with_metaclass(ABCMeta)):
    def get_template_part(self, part, lang=None):

        templates = []
        # 1/ localized: mails/invitation_code/fr/
        localized = os.path.join('sms', self.template_name, lang or self.lang, part)
        templates.append(localized)

        # 2/ fallback: mails/invitation_code/
        fallback = os.path.join('sms', self.template_name, part)
        templates.append(fallback)

        # return the list of templates path candidates
        return templates

    def mail_admins(self, attachments=None, from_email=None):
        raise NotImplementedError()

    def create_sms_msg(self, lang=None):
        try:
            body = self._render_part('body.txt', lang=lang)
        except TemplateDoesNotExist:
            raise TemplateDoesNotExist(
                'Txt template have not been found')
        return body

    @abstractmethod
    def send(self, to_phone, from_phone=None):
        return



# coding=utf-8

from django.test import TestCase
from django.core import mail
from django.conf import settings

from mail_factory.exceptions import MissingMailContextParamException

from mailfactory_extras.backends.celery import (
    CeleryMail, AsyncEmailMultiRelated)


class CeleryMailTest(TestCase):

    def __init__(self, *args, **kwargs):
        super(CeleryMailTest, self).__init__(*args, **kwargs)
        self.mail_class = CeleryMail

    def test_init(self):
        class TestMail(self.mail_class):
            params = ['foo']

        with self.assertRaises(MissingMailContextParamException):
            TestMail()
        with self.assertRaises(MissingMailContextParamException):
            TestMail({})
        with self.assertRaises(MissingMailContextParamException):
            TestMail({'bar': 'bar'})
        self.assertTrue(TestMail({'foo': 'bar'}))

    def test_get_language(self):
        class TestMail(self.mail_class):
            params = []

        translation.activate('en')
        self.assertEqual(TestMail().get_language(), 'en')
        translation.activate('fr')
        self.assertEqual(TestMail().get_language(), 'fr')

    def test_get_params(self):
        class TestMail(self.mail_class):
            params = []

        self.assertEqual(TestMail().get_params(), [])

        class TestMail2(self.mail_class):
            params = ['foo']

        self.assertEqual(TestMail2({'foo': 'bar'}).get_params(), ['foo'])

    def test_get_context_data(self):
        class TestMail(self.mail_class):
            params = []

        self.assertEqual(TestMail().get_context_data(), {})
        self.assertEqual(TestMail().get_context_data(foo='bar'),
                         {'foo': 'bar'})

    def test_get_attachments(self):
        class TestMail(self.mail_class):
            params = []

        self.assertEqual(TestMail().get_attachments(None), [])
        self.assertEqual(TestMail().get_attachments('foo'), 'foo')

    def test_get_template_part(self):
        class TestMail(self.mail_class):
            params = []
            template_name = 'test'

        test_mail = TestMail()
        test_mail.lang = 'foo'
        self.assertEqual(test_mail.get_template_part('stuff'),
                         ['mails/test/foo/stuff', 'mails/test/stuff'])
        self.assertEqual(test_mail.get_template_part('stuff', 'bar'),
                         ['mails/test/bar/stuff', 'mails/test/stuff'])

    def test_render_part(self):
        class TestMail(self.mail_class):
            params = []
            template_name = 'test'

        test_mail = TestMail()
        with self.assertRaises(TemplateDoesNotExist):
            test_mail._render_part('stuff')

        # active language before stays the same
        cur_lang = translation.get_language()
        test_mail._render_part('subject.txt')
        self.assertEqual(cur_lang, translation.get_language())

        # without a proper language, fallback
        test_mail.lang = 'not a lang'
        self.assertEqual(test_mail._render_part('subject.txt'),
                         '[TestCase] Mail test subject')
        # with a proper language, use it
        test_mail.lang = 'fr'
        self.assertEqual(test_mail._render_part('body.txt'), 'Français')
        # use provided language
        test_mail.lang = 'not a lang'
        self.assertEqual(test_mail._render_part('subject.txt', 'en'),
                         '[TestCase] Mail test subject')
        self.assertEqual(test_mail._render_part('body.txt', 'fr'), 'Français')
        # if provided language doesn't exist, fallback
        self.assertEqual(test_mail._render_part('subject.txt', 'not a lang'),
                         '[TestCase] Mail test subject')

    def test_create_email_msg(self):
        class TestMail(self.mail_class):
            params = []
            template_name = 'test'

        test_mail = TestMail()
        # check is using the custom message class
        self.assertIsInstance(
            test_mail.create_email_msg([]), CeleryEmailMultiRelated)

        # no "from email" given => use settings.DEFAULT_FROM_EMAIL
        self.assertEqual(test_mail.create_email_msg([]).from_email,
                         settings.DEFAULT_FROM_EMAIL)
        msg = test_mail.create_email_msg([], from_email='foo')
        self.assertEqual(msg.from_email, 'foo')
        # templates with html
        msg = test_mail.create_email_msg([], lang='fr')
        self.assertEqual(len(msg.alternatives), 1)
        msg = test_mail.create_email_msg([], lang='en')
        self.assertEqual(len(msg.alternatives), 1)
        # templates without html
        test_mail.template_name = 'test_no_html'
        msg = test_mail.create_email_msg([], lang='fr')
        self.assertEqual(len(msg.alternatives), 0)
        msg = test_mail.create_email_msg([], lang='en')
        self.assertEqual(len(msg.alternatives), 0)
        #template without txt
        test_mail.template_name = 'test_no_txt'
        self.assertEqual(test_mail.create_email_msg(
            emails=['receiver@mail.com', ],
            from_email="receiver@mail.com", lang='fr').body,
            '# Français\n\n')
        #template without html and without txt
        test_mail.template_name = 'test_no_html_no_txt'

        with self.assertRaises(TemplateDoesNotExist):
            test_mail.create_email_msg(
                emails=['receiver@mail.com', ],
                from_email="receiver@mail.com", lang='fr')

    def test_create_email_msg_attachments(self):
        class TestMail(self.mail_class):
            params = []
            template_name = 'test'

        test_mail = TestMail()
        attachments = [
            (finders.find('admin/img/nav-bg.gif'), 'nav-bg.gif', 'image/png'),
            (finders.find('admin/css/base.css'), 'base.css', 'plain/text')]
        msg = test_mail.create_email_msg([], attachments=attachments)
        self.assertEqual(len(msg.attachments), 1) # base.css
        self.assertEqual(len(msg.related_attachments), 1) # nav-bg.gif

    def test_send(self):
        class TestMail(self.mail_class):
            params = []
            template_name = 'test'

        before = len(mail.outbox)

        task = TestMail().send(['foo@bar.com'])
        self.assertTrue(task.successful())
        self.assertEqual(task.get(), None)
        self.assertEqual(len(mail.outbox), before + 1)
        self.assertEqual(mail.outbox[-1].to, ['foo@bar.com'])




# coding=utf-8

from django.conf import settings

from ..base import BaseSMS
from twilio.rest import TwilioRestClient


class TwilioSMS(BaseSMS):
    def send(self, to_phone, from_phone=None):
        from_phone = from_phone or settings.TWILIO_PHONE_SERVER
        message = self.create_sms_msg()
        client = TwilioRestClient(
            settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN)
        response = client.messages.create(
            to=to_phone, from_=from_phone, body=message)
        return response




# coding=utf-8

from mail_factory.mails import BaseMail

from .messages import CeleryEmailMultiRelated, CeleryEmailMultiRelatedMetric


class CeleryMail(BaseMail):
    def __init__(self, *args, **kwargs):
        """Add async option to BaseMail

            :param async: True to send using task

        """
        self.async = kwargs.pop('async', True)
        super(CeleryMail, self).__init__(*args, **kwargs)

    def create_email_msg(
            self, emails, attachments=None, from_email=None, lang=None,
            message_class=CeleryEmailMultiRelated):
        return super(CeleryMail, self).create_email_msg(
            emails, attachments=attachments, from_email=from_email,
            lang=lang, message_class=message_class)

    def send(self, emails, attachments=None, from_email=None):
        message = self.create_email_msg(
            emails, attachments=attachments, from_email=from_email)
        return message.send(fail_silently=False, async=self.async)


class CeleryMailMetric(CeleryMail):
    def create_email_msg(
            self, emails, attachments=None, from_email=None, lang=None,
            message_class=CeleryEmailMultiRelatedMetric):
        return super(CeleryMailMetric, self).create_email_msg(
            emails, attachments=attachments, from_email=from_email,
            lang=lang, message_class=message_class)




# coding=utf-8

from celery import shared_task
from celery.contrib.methods import task_method

from mail_factory.messages import EmailMultiRelated

try:
    from app_metrics.utils import metric
except ImportError:
    metric = None


class CeleryEmailMultiRelated(EmailMultiRelated):
    @shared_task(
        name='AsyncEmailMultiRelated._send_async', filter=task_method)
    def _send(self, fail_silently=False):
        """With this decorator, celery will support calling class
        methods as tasks

        """
        return super(EmailMultiRelated, self).send()

    def send(self, fail_silently=False, async=False):
        """Check if mail will be sent using async task or not

        """
        if async:
            return self._send.delay(fail_silently=fail_silently)
        return self._send(fail_silently=fail_silently)


class CeleryEmailMultiRelatedMetric(CeleryEmailMultiRelated):
    """Metric are handled via django-app-metrics

    """
    waiting_metric = 'mails_waiting'
    sended_metric = 'mails_sended'

    @shared_task(
        name='AsyncEmailMultiRelatedMetric._send_async', filter=task_method)
    def _send(self, fail_silently=False):
        """With this decorator, celery will support calling class
        methods as tasks

        """
        result = super(EmailMultiRelated, self).send()
        if metric is not None:
            metric(self.sended_metric)
            metric(self.waiting_metric, -1)
        return result

    def send(self, fail_silently=False, async=False):
        if metric is not None:
            metric(self.waiting_metric)
        return super(CeleryEmailMultiRelatedMetric, self).send(
            fail_silently=fail_silently, async=async)




# coding=utf-8

try:
    from django.apps import AppConfig
except ImportError:
    AppConfig = object

class MailFactoryExtrasConfig(AppConfig):
    name = 'django_mailfactory_extras'
    verbose = 'Django MailFactory extras'




# coding=utf-8

import os
from setuptools import setup, find_packages
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

import mailfactory_extras as me

setup(
    name='django-mailfactory-extras',
    version=me.__version__,
    url='https://github.com/jneight/django-mailfactory-extras',
    install_requires=['django-mail-factory'],
    description="Extensions for django-mail-factory, like django-xadmin integration or asynchronous message sending",
    author=me.__author__,
    author_email=me.__email__,
    packages=find_packages(exclude=['tests', 'tests.*',]),
    include_package_data=True,
    license=me.__license__,
    test_suite="tests",
    zip_safe=False,
)
